import type { ConnectUIProps } from './connectUI';
import { ConnectUI } from './connectUI.js';
import type { ConnectionConfig, AuthErrorType, AuthOptions, AuthResult, ErrorHandler } from './types';
export type * from './types';
export * from './connectUI.js';
export type NangoOptions = {
    host?: string;
    websocketsPath?: string;
    width?: number;
    height?: number;
    debug?: boolean;
} & ({
    connectSessionToken?: string;
    publicKey?: never;
} | {
    connectSessionToken?: never;
    publicKey?: string;
});
export declare class AuthError extends Error {
    type: AuthErrorType;
    constructor(message: string, type: AuthErrorType);
}
export default class Nango {
    private hostBaseUrl;
    private websocketsBaseUrl;
    private status;
    private publicKey;
    private connectSessionToken;
    private debug;
    private width;
    private height;
    private tm;
    win: AuthorizationModal | null;
    constructor(config?: NangoOptions);
    /**
     * Creates a new unauthenticated connection using the specified provider configuration key and connection ID
     * @param providerConfigKey - The key identifying the provider configuration on Nango
     * @param connectionId -  Optional. The ID of the connection
     * @param connectionConfig - Optional. Additional configuration for the connection
     * @returns A promise that resolves with the authentication result
     */
    create(providerConfigKey: string, connectionConfig?: ConnectionConfig): Promise<AuthResult>;
    create(providerConfigKey: string, connectionId: string, connectionConfig?: ConnectionConfig): Promise<AuthResult>;
    /**
     * Initiates the authorization process for a connection
     * @param providerConfigKey - The key identifying the provider configuration on Nango
     * @param connectionId - Optional. The ID of the connection for which to authorize
     * @param options - Optional. Additional options for authorization
     * @returns A promise that resolves with the authorization result
     */
    auth(providerConfigKey: string, options?: AuthOptions): Promise<AuthResult>;
    auth(providerConfigKey: string, connectionId: string, options?: AuthOptions): Promise<AuthResult>;
    reconnect(providerConfigKey: string, options?: AuthOptions): Promise<AuthResult>;
    /**
     * Clear state of the frontend SDK
     */
    clear(): void;
    /**
     * Open managed Connect UI
     */
    openConnectUI(params: ConnectUIProps): ConnectUI;
    /**
     * Converts the provided credentials to a Connection configuration object
     * @param credentials - The credentials to convert
     * @returns The connection configuration object
     */
    private convertCredentialsToConfig;
    private triggerAuth;
    /**
     * Performs authorization based on the provided credentials i.e api, basic, appstore and oauth2
     * @param providerConfigKey - The key identifying the provider configuration on Nango
     * @param connectionId - The ID of the connection for which to create the custom Authorization
     * @param connectionConfigWithCredentials - The connection configuration containing the credentials
     * @param connectionConfig - Optional. Additional connection configuration
     * @returns A promise that resolves with the authorization result
     */
    private customAuth;
    /**
     * Converts the connection ID and configuration parameters into a query string
     * @param connectionId - The ID of the connection for which to generate a query string
     * @param connectionConfig - Optional. Additional configuration for the connection
     * @returns The generated query string
     */
    private toQueryString;
    /**
     * Check that we have one valid credential
     * It's not done in the constructor because if you only use Nango Connect it's not relevant to throw an error
     */
    private ensureCredentials;
}
/**
 * AuthorizationModal class
 */
declare class AuthorizationModal {
    private url;
    private features;
    private width;
    private height;
    private swClient;
    private debug;
    private wsClientId;
    private errorHandler;
    modal: Window | undefined;
    isProcessingMessage: boolean;
    constructor(webSocketUrl: string, url: string, successHandler: (providerConfigKey: string, connectionId: string) => any, errorHandler: ErrorHandler, { width, height }: {
        width?: number | null;
        height?: number | null;
    }, debug?: boolean);
    /**
     * Handles the messages received from the Nango server via WebSocket
     * @param message - The message event containing data from the server
     * @param successHandler - The success handler function to be called when a success message is received
     */
    handleMessage(message: MessageEvent, successHandler: (providerConfigKey: string, connectionId: string) => any): void;
    /**
     * Calculates the layout dimensions for a modal window based on the expected width and height
     * @param expectedWidth - The expected width of the modal window
     * @param expectedHeight - The expected height of the modal window
     * @returns The layout details including left and top positions, as well as computed width and height
     */
    layout(expectedWidth: number, expectedHeight: number): {
        left: number;
        top: number;
        computedWidth: number;
        computedHeight: number;
    };
    /**
     * Opens a modal window with the specified WebSocket client ID
     */
    open(): void;
    /**
     * Converts the features object of this class to a string
     * @returns The string representation of features
     */
    featuresToString(): string;
    /**
     * Close modal, if opened
     */
    close(): void;
}
//# sourceMappingURL=index.d.ts.map